<?php

use Nette\Forms\Form;
use Instante\Bootstrap3Renderer\BootstrapRenderer;

function render_match_action($action) {
    return function($id) use ($action) {
        global $app;
        $app->render("match/$action", array('match' => find_match($id)));
    };
}

function filter_form($matches, $user_id=null) {
    global $db;

    $match_ids = array_unique($matches->fetchPairs(null, 'id'));
    $tags = $db->table('tag')
        ->select('DISTINCT name AS name')
        ->where(array('match_id' => $match_ids))
        ->fetchPairs('name', 'name');
    asort($tags);

    $distances = $matches->fetchPairs('distance', 'distance');
    asort($distances);
    $distances = map_format(_('%d meters'), $distances);

    $form = new Form;
    $form->setRenderer(new BootstrapRenderer);
    $form->setAction(($user_id ? 'user/' . $user_id . '/' : '') . 'matches#results');
    $form->setMethod('get');

    $form->addSelect('distance', _('Distance'), $distances)
        ->setPrompt(_('Select a distance'));
    $form->addSelect('tag', _('Tag'), $tags)
        ->setPrompt(_('Select a tag'));
    $form->addText('name', _('Name'));
    $form->addSubmit('send', _('Filter'));

    return $form;
}

function create_form() {
    global $db, $user;

    $disciplines = array('barebow', 'recurve', 'compound');
    $disciplines = array_combine($disciplines, array_map('ucfirst', $disciplines));

    $form = new Form;
    $form->setRenderer(new BootstrapRenderer);
    $form->setAction('match');

    $form->addText('name', _('Name'))
        ->setRequired();
    $form->addText('created_at', _('Date'))
        ->setType('datetime-local')
        ->setDefaultValue(strftime('%Y-%m-%dT%H:%M'))
        ->setAttribute('placeholder', _('YYYY-MM-DD HH:MM'))
        ->addRule(Form::PATTERN, _('Invalid date'), '\d{4}-\d{1,2}-\d{1,2}[T ]\d{1,2}:\d{1,2}')
        ->setRequired();
    $form->addText('distance', _('Distance'))
        ->setAttribute('placeholder', _('Distance in meters'))
        ->setType('number')
        ->setRequired();
    $form->addRadioList('discipline', _('Discipline'), $disciplines)
        ->setAttribute('data-inline', true)
        ->setRequired();
    $form->addText('arrows', _('Arrows'))
        ->setType('range')
        ->setAttribute('min', 1)
        ->setAttribute('max', 10)
        ->setDefaultValue(3)
        ->setRequired();
    $form->addText('turns', _('Turns'))
        ->setType('number')
        ->setAttribute('min', 1)
        ->setDefaultValue(10)
        ->setRequired();
    $form->addTextarea('notes', _('Notes'));
    $form->addSubmit('send', _('Create'));

    // Set attributes from last match as default, since the shooter is likely
    // to repeat the previous training in many cases
    $last_match = $db->table('match')
        ->where(array('user_id' => $user->getId()))
        ->order('created_at DESC')
        ->fetch();

    if ($last_match) {
        //$form['name']->setDefaultValue($last_match->name);
        $form['distance']->setDefaultValue($last_match->distance);
        $form['discipline']->setDefaultValue($last_match->discipline);
        $form['arrows']->setDefaultValue($last_match->arrows);
        $form['turns']->setDefaultValue($last_match->turns);
    }

    return $form;
}

function edit_form($match) {
    $form = create_form();

    $form->setAction("match/{$match->id}");
    $form->addHidden('_METHOD', 'PUT');

    $created_at = preg_replace('/^(\d{4}-\d{1,2}-\d{1,2})[T ](\d{1,2}:\d{1,2}):\d{1,2}$/',
                               '\1T\2', $match->created_at);

    $form['name']->setDefaultValue($match->name);
    $form['created_at']->setDefaultValue($created_at);
    $form['distance']->setDefaultValue($match->distance);
    $form['discipline']->setDefaultValue($match->discipline);
    $form['arrows']->setDefaultValue($match->arrows);
    $form['turns']->setDefaultValue($match->turns);
    $form['notes']->setDefaultValue($match->notes);

    return $form;
}

function create_tag($match, $tagname) {
    global $db;

    // Need to catch the "unknown column tag.id" error because
    // primary key fetching is bugged
    try {
        $db->table('tag')->insert(array(
            'name' => $tagname,
            'match_id' => $match->id
        ));
    } catch(PDOException $e) {}
}

function user_matches($user_id, $admin_view, $page) {
    global $config, $app, $db;

    $matches = $db->table('match')
        ->where(array('user_id' => $user_id))
        ->order('created_at DESC');

    // Filtering
    $filter_form = filter_form($matches, $admin_view ? $user_id : null);

    if ($filter_form->isSubmitted()) {
        $values = $filter_form->getValues();

        if ($values->tag) {
            $ids = $db->table('tag')->select('match_id')->where('name', $values->tag);
            $matches->where('id', $ids);
        }

        if ($values->distance)
            $matches->where('distance', $values->distance);

        if ($values->name)
            $matches->where('name LIKE ?', "%$values->name%");
    }

    // Pagination
    $pagesize = $config['max_list_items'];
    $nresults = $matches->count();
    $npages = intval(ceil($nresults / floatval($pagesize)));
    $matches->limit($pagesize, ($page - 1) * $pagesize);

    $dbuser = $admin_view ? find_user($user_id) : null;
    $base_uri = $admin_view ? "user/$user_id/matches" : 'matches';
    $app->render('match/list', compact(
        'matches', 'filter_form', 'dbuser', 'page', 'npages', 'nresults',
        'base_uri'
    ));
}

$app->get('/matches(/:page)', function ($page=1) use ($user) {
    return user_matches($user->getId(), false, max(1, intval($page)));
});
$app->get('/match/:id', render_match_action('view'));
$app->get('/match/:id/scores', render_match_action('scores'));
$app->get('/match/:id/delete', render_match_action('delete'));

$app->get('/match', function () use ($app) {
    $app->render('match/create', array('form' => create_form()));
});
$app->post('/match', function () use ($app, $db, $user) {
    $form = create_form();
    $form->validate();

    if (!$form->hasErrors()) {
        $values = $form->getValues();

        $values->created_at = str_replace('T', ' ', $values->created_at) . ':00';
        $values->scores = pack_scores(array_fill(0, $values->turns * $values->arrows, 0));
        $values->user_id = $user->getId();

        $match = $db->table('match')->insert($values);

        if (isset($_POST['tags'])) {
            foreach (array_unique($_POST['tags']) as $tagname)
                create_tag($match, $tagname);
        }

        $app->redirect(ROOT_URL . "/match/{$match->id}/scores");
    }

    $app->render('match/create', compact('form'));
});

$app->post('/match/:id/scores', function ($id) use ($app, $db) {
    find_match($id)->update(array(
        'scores' => pack_scores(array_map('intval', $_POST['scores'])),
        'notes' => $_POST['notes']
    ));
    $app->redirect(ROOT_URL . '/match/' . $id);
});

$app->delete('/match/:id', function ($id) use ($app, $db) {
    find_match($id)->delete();
    $app->redirect(ROOT_URL . '/matches');
});

$app->get('/match/:id/edit', function ($id) use ($app) {
    $match = find_match($id);
    $form = edit_form($match);
   $app->render('match/edit', compact('form', 'match'));
});

$app->put('/match/:id', function ($id) use ($app, $db) {
    function zeropad($array, $size) {
        return array_pad(array_slice($array, 0, $size), $size, 0);
    }

    $match = find_match($id);
    $form = edit_form($match);
    $form->validate();

    if (!$form->hasErrors()) {
        $values = $form->getValues();
        $update = array(
            'name' => $values->name,
            'created_at' => str_replace('T', ' ', $values->created_at) . ':00',
            'distance' => $values->distance,
            'discipline' => $values->discipline,
            'arrows' => $values->arrows,
            'turns' => $values->turns,
            'notes' => $values->notes
        );

        if ($values->arrows != $match->arrows || $values->turns != $match->turns) {
            $scores = unpack_scores($match->scores);
            $turns = array_chunk($scores, $match->arrows);

            foreach ($turns as $i => $turn)
                $turns[$i] = zeropad($turn, $values->arrows);

            $scores = call_user_func_array('array_merge', $turns);
            $scores = zeropad($scores, $values->turns * $values->arrows);
            $update['scores'] = pack_scores($scores);
        }

        $match->update($update);

        $tagnames = isset($_POST['tags']) ? $_POST['tags'] : array();
        $old_tags = $match->related('tag');
        $old_tagnames = $old_tags->fetchPairs(null, 'name');
        $add = array_diff($tagnames, $old_tagnames);
        $remove = array_values(array_diff($old_tagnames, $tagnames));

        $db->table('tag')->where('name', $remove)->delete();

        foreach ($add as $tagname)
            create_tag($match, $tagname);

        $app->redirect(ROOT_URL . "/match/{$match->id}");
    }

    $app->render('match/edit', compact('form', 'match'));
});
